import QtQuick 2.0
import QtQuick.XmlListModel 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import Ubuntu.Components.Popups 0.1

import "../databasemodule_v2.js" as DB

Page {
    id: manageTopicsPage

    title: i18n.tr("Topics")
    flickable: null

    tools: ToolbarItems {

        ToolbarButton {
            action: Action {
                text: "Add topic"
                iconSource: Qt.resolvedUrl("../icons_tmp/add.svg")

                onTriggered: {
                    pageStack.push(createTopicPage)
                }
            }
        }
    }

    function reloadPageContent() {
        manageTopicsListModel.clear()

        var tags = DB.loadTags()

        for (var i = 0; i < tags.rows.length; i++) {
            var curTag = tags.rows.item(i)

            manageTopicsListModel.append({"tagName" : curTag.name, "tagId" : curTag.id})
        }
    }

    ListView {
        id: manageTopicsList
        objectName: "topiclist"
        anchors.fill: parent
        clip: true
        model: manageTopicsListModel

        delegate: ListItem.Standard {
            id: topicDelegate
            objectName: "topicDelegate"
            text: model.tagName
            removable: true

            Item {
                id: removeTopicIndicator
                objectName: "topicItem"
                //state: topicDelegate.swipingState === "" ? "hidden" : "visible"
                anchors.fill: parent
                Rectangle {
                    objectName: "topicRect"
                    opacity: 0.8
                    anchors.fill: parent
                }

                Label {
                    objectName: "topiclabel"
                    anchors.fill: parent
                    color: UbuntuColors.orange
                    anchors.margins: units.gu(4)
                    text: "Swipe to remove topic"
                    //horizontalAlignment: topicDelegate.swipingState === "SwipingLeft" ? Text.AlignRight : Text.AlignLeft
                    horizontalAlignment: Text.AlignHCenter
                    verticalAlignment: Text.AlignVCenter
                }

                opacity: topicDelegate.swipingState === "" ? 0 : 1

                Behavior on opacity {
                    NumberAnimation {}
                }
            }

            backgroundIndicator: Rectangle {
                anchors.fill: parent
                color: UbuntuColors.warmGrey//"gray"
            }

            onItemRemoved: {
                console.log("ITEM REMOVED", manageTopicsListModel.get(index).tagName)
                var tagId = manageTopicsListModel.get(index).tagId
                DB.deleteFeedTagsByTagId(tagId)
                DB.deleteTag(tagId)
            }

            onPressAndHold: {
                manageTopicsList.currentIndex = model.index
                PopupUtils.open(manageTopicsPopover, manageTopicsList.currentItem)
            }
        }
    }

    Component {
        id: manageTopicsPopover

        /* This new popover works like context menu.
                 * Actions can be reused in toolbar.
                 */
        ActionSelectionPopover {
            id: manageTopicsActionSelector
            actions: ActionList {
                id: popoverActionsList

                Action {
                    text: i18n.tr("Edit")
                    onTriggered: {
                        PopupUtils.close(manageTopicsActionSelector)
                        console.log("Edit topic name TODO")
                    }
                }

                Action {
                    text: i18n.tr("Delete with feeds")
                    onTriggered: {
                        PopupUtils.close(manageTopicsActionSelector)
                        console.log("Delete with feeds TODO")
                    }
                }
            } // ActionList
        } // ActionSelectionPopover
    } // Popover component

    ListModel {
        id: manageTopicsListModel
    }

    Label {
        visible: manageTopicsListModel.count == 0
        anchors.centerIn: parent
        text: i18n.tr("Use toolbar to append topics")
        fontSize: "large"
    }
} // Page "Topics"
