import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem

import "../databasemodule_v2.js" as DB

ListItem.Empty {
    id: topicComponent

    height: columnContent.height
//    removable: true
//    highlightWhenPressed : true
    showDivider : true
//    clip: true

    property bool isExpended: false
    property string topicName
    property int topicId
    property int modelIndex

//    onClicked: isExpended = !isExpended ;

    onStateChanged:{
        if (isExpended){
            reloadFeed()
        }
    }

    Behavior on height {
        NumberAnimation { duration: 150; easing.type: Easing.OutQuad }
    }

    function reloadFeed (){
        feedModel.clear()
        var feeds_tag =  DB.loadFeedsFromTag(topicId)
        for (var i=0; i< feeds_tag.rows.length; i++){
            feedModel.append(feeds_tag.rows[i]) ;
        }
    }

    Column{
        id: columnContent
        anchors{ left: parent.left; right: parent.right }

        Item
        {
            anchors{ left: parent.left; right: parent.right }
            height: units.gu(5)

            ListItem.Base{
                id: headerTopic
                removable: true
                height: units.gu(5)

                onClicked: isExpended = !isExpended

                onItemRemoved:{
                    console.log("item about to be removed: ", topicComponent.topicId)
//                    itemCancelDelete.startCounting()
                    var result = DB.deleteFeedByTagId(topicComponent.topicId)
                    if (result.rowsAffected == undefined)
                    {
                        topicManagement.reloadTopics()
                    }
                    result = DB.deleteTag(topicComponent.topicId)
//                            topicManagement.reloadTopics()
                    if (result.rowsAffected == 1)
                    {
                        topicManagement.removeModelItem(topicComponent.modelIndex)
                    }
                    else
                    {
                        topicManagement.reloadTopics()
                    }
                }

                Row{
                    id: rowTopicContent
                    anchors
                    {
                        top: parent.top; bottom: parent.bottom; left: parent.left;
                        leftMargin: units.gu(0); topMargin: units.gu(1); bottomMargin: units.gu(1);
                    }
                    spacing: units.gu(2)

                    property bool isEditing: false

                    Image{
                        anchors
                        {
                            top: parent.top; bottom: parent.bottom;
                            topMargin: units.gu(0.5); bottomMargin: units.gu(0.5);
                        }
                        fillMode: Image.PreserveAspectFit
                        source: Qt.resolvedUrl("../icons_tmp/compose.svg")
                        smooth: true

                        MouseArea{
                            anchors { fill: parent; margins: units.gu(-1) }
                            onClicked:
                            {
                                if (rowTopicContent.isEditing)
                                {
                                    var result = DB.updateTag(topicComponent.topicId, inputTopicName.text)
                                    if (result.rowsAffected == 1)
                                    {
                                        labelTopicName.text = inputTopicName.text
                                    }
                                    else
                                    {
                                        inputTopicName.text = labelTopicName.text
                                    }
                                }
                                else
                                {}
                                rowTopicContent.isEditing = !rowTopicContent.isEditing
                            }
                        }
                    }

                    Label{
                        id: labelTopicName
                        anchors.verticalCenter: parent.verticalCenter
                        text: topicName
                        width: rowTopicContent.isEditing ? 0 : paintedWidth
                        opacity: rowTopicContent.isEditing ? 0 : 1

    //                    Behavior on width { UbuntuNumberAnimation{} }
                    }

                    TextField{
                        id: inputTopicName
                        anchors.verticalCenter: parent.verticalCenter
                        text: topicName
                        width: rowTopicContent.isEditing ? headerTopic.width - units.gu(15) : 0
                        opacity: rowTopicContent.isEditing ? 1 : 0
                        hasClearButton: true
                        activeFocusOnPress: true

                        Behavior on width { UbuntuNumberAnimation{} }

                        Connections {
                            target: Qt.inputMethod
                            onVisibleChanged: {
                                if (!Qt.inputMethod.visible) {
                                    topicComponent.focus = true
                                }
                            }
                        }
                    }
                }

                Image{
                    id: imgArrow
                    anchors
                    {
                        right: parent.right; top: parent.top; bottom: parent.bottom;
                        topMargin: units.gu(1.5); bottomMargin: units.gu(1.5); rightMargin: units.gu(2)
                    }
                    fillMode: Image.PreserveAspectFit
                    source: Qt.resolvedUrl("../icons_tmp/go-to.svg")
                    smooth: true

                    Behavior on rotation { UbuntuNumberAnimation{} }

                    states: [
                        State {
                            name: "expended"
                            when: topicComponent.isExpended

                            PropertyChanges
                            {
                                target: imgArrow
                                rotation: 90
                            }
                        }
                    ]
                }
            }

        }

        ListView
        {
            id: feedList
            anchors{ left: parent.left; right: parent.right }
            height: 0
            opacity: 0
            interactive: false

            Behavior on opacity {
                PropertyAnimation { duration: 150; easing.type: Easing.OutQuad }
            }

            model: VisualDataModel {
                id: visualModel
                model: ListModel {
                    id: feedModel
                }
                delegate: MouseArea {
                    id: delegateFeed

                    property int visualIndex: VisualDataModel.itemsIndex
                    property int feedId: model.id
                    property int topicId: topicComponent.topicId

//                    anchors{ left: parent.left; right: parent.right }
                    width: feedList.width
                    height: feedItem.height

                    function pressHold()
                    {
                        console.log("press and hold")
                        delegateFeed.drag.target = feedItem ;
                        feedItem.opacity = 0.5 ;
//                        topicComponent.isExpended = false ;
                        topicList.collapseAll() ;
                    }

                    onPressAndHold:
                    {
                        pressHold() ;
                    }

                    onClicked:
                    {
                        mainView.toEditFeed(model.id, model.title, model.source, delegateFeed.topicId) ;
                    }

                    onReleased:
                    {
//                        console.log("release drag")
                        feedItem.Drag.drop() ;
                        feedItem.Drag.active = false ;
                    }

                    FeedComponent
                    {
                        id: feedItem
                        text: model.title
                        feedId: model.id
                        topicId: delegateFeed.topicId
                        width: parent.width
                        height: units.gu(6)
//                        removable: true

                        Drag.active: delegateFeed.drag.active
                        Drag.source: delegateFeed

                        onPressHold:
                        {
                            parent.pressHold() ;
                        }
                    }

                    states: [
                        State {
                            name: "draging"
                            when: feedItem.Drag.active
                            ParentChange {
                                target: feedItem
                                parent: topicManagement
                            }

                            AnchorChanges {
                                target: feedItem;
                                anchors.horizontalCenter: undefined;
                                anchors.verticalCenter: undefined
                            }
                        },

                        State
                        {
                            name: ""
                            ParentChange {
                                target: feedItem
                                parent: delegateFeed
                            }

//                            AnchorChanges {
//                                target: feedItem;
//                                anchors.left: delegateFeed.left
//                                anchors.top: delegateFeed.top
//                            }
                        }

                    ]
                }
            }
        }
    }



    states: [
        State {
            name: "expended"
            when: isExpended
            PropertyChanges
            {
                target: feedList
                height: feedList.contentItem.childrenRect.height
                opacity: 1
            }
        },

        State {
            name: ""
//            when: isExpended
            PropertyChanges
            {
                target: feedList
                height: 0
                opacity: 0
            }
        }
    ]
}

