import QtQuick 2.0
import QtQuick.XmlListModel 2.0

import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItems
import Ubuntu.Components.Popups 0.1

import "../databasemodule_v2.js" as DB

Page {
    objectName: "articlelistpage"
    signal articleSelected(var data) // mb id.
    property var updateData: null

    function reloadPageContent() {
        loadArticles(updateData)
        title = updateData.title
    }

    /* 1. Loading data from database by url.
     * 2. Appending data to ListModel for displaying.
     */
    function loadArticles(params) {
        articleList.model = null // TMP - doesn't work properly without that, mb bug.
        articleListModel.clear()

        var res = DB.loadArticles(params)
        var entries = res.rows

        for (var i = 0; i < entries.length; i++) {
            var e = entries.item(i)
            articleListModel.append({"title" : e.title,
                                        "content" : e.content,
                                        "link" : e.link,
                                        "description" : e.description,
                                        "pubdate" : e.pubdate,
                                        "status" : e.status,
                                        "favourite" : e.favourite,
                                        "feed_id" : e.feed_id,
                                        "image" : e.image,
                                        "feed_name" : e.feed_name,
                                        "guid" : e.guid}) // for what?
        }

        articleList.model = articleListModel // TMP - doesn't work properly without that, mb bug.
        articleListToolbar.opened = false
    }

    tools: ToolbarItems {
        id: articleListToolbar
    }

    ListView {
        id: articleList

        clip: true
        anchors.fill: parent

        model: articleListModel

        delegate: ListItems.Subtitled {
            text: model.title
            subText: model.pubdate
            progression: true
            icon: selectIcon()
            iconFrame: true

            function selectIcon() {
                var img = model.image
                // console.log("DELEGATE IMG", img)
                return (model.image == null || model.image == "" ) ? Qt.resolvedUrl("../rssreader64.png"): model.image
            }

            onPressAndHold: {
                articleList.currentIndex = model.index
            }

            onClicked: {
                console.log("ARTICLE SELECTED", model.title)
                // TODO DEMO
                mainView.toRssPage(articleListModel, model.index)
            }
        } // delegate
    } // ListView

    ListModel {
        id: articleListModel
    }

    Label {
        id: lblEmptyBase

        text: i18n.tr("There are no articles in selected feed")
        anchors.centerIn: parent
        visible: articleListModel.count == 0
        fontSize: "large"
    }
}
