import QtQuick 2.0
import QtQuick.XmlListModel 2.0

import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItems
import Ubuntu.Components.Popups 0.1

import "../databasemodule_v2.js" as DB
import "../dateutils.js" as DateUtils
import "../."

Page {
    id: pageItself

    property bool isDirty: false // Need to clear all page content.

    property int topicId: 0
    property bool isShorts: topicId == 0

    flickable: isShorts ? articleList : articleListByTag

    function reload() {
        isDirty = true
        reloadPageContent()
    }

    function reloadPageContent() {
        if (!isDirty) {
            console.log("IS DIRTY FALSE")
            return
        }
        isDirty = false

        // TODO mb usable code. Need to discuss it.
        // articleList.model = null
        // articleListByTag.model = null

        articleListModel.clear()
        articleListProxyModel.clear()

        var tagArray = loadTags()

        var global_number = 0

        for (var i = 0; i < tagArray.length; i++) {
            var curTag = tagArray[i]

            var tagArticles = DB.loadArticles({"tagId" : curTag.id})
            console.log("THERE ARE", tagArticles.rows.length, "ARTICLES IN TAG", curTag.id, curTag.name)

            var articlesByTag = []
            for (var j = 0; j < tagArticles.rows.length; j++) {
                var ca = tagArticles.rows.item(j)

                var articleObj = {"tagName" : curTag.name,
                    "tagId" : curTag.id,
                    "title" : ca.title,
                    "content" : ca.content,
                    "link" : ca.link,
                    "description" : ca.description, // ?
                    "pubdate" : ca.pubdate,
                    "status" : ca.status, // ?
                    "favourite" : ca.favourite,
                    "feedId" : ca.feed_id,
                    "image" : ca.image,
                    "feed_name" : ca.feed_name,
                    "guid" : ca.guid, // ?
                    "serialNumber" : global_number++}

                articlesByTag.push(articleObj)
                articleListProxyModel.append(articleObj)
            }

            /* Little randomization of results.
             * Issue (or feature :D) - each time user switches to list mode
             * he will see different articles.
             * TODO Replace with sort by date.
             */
            var k = 0
            var limit = Math.min(articlesByTag.length, 8)
            while (k < limit) {
                var index = Math.floor(Math.random() * 0x12345678) % articlesByTag.length
                var obj = articlesByTag[index]
                articleListModel.append(obj)
                articlesByTag.splice(index, 1)
                k++
            }
        }

        // TODO mb usable code. Need to discuss it.
        //articleList.model = isShorts ? articleListModel : null
        //articleListByTag.model = isShorts ? null : articleListProxyModel

    } // reloadPageContent

    function loadTags() {
        if (isShorts) {
            var tags = DB.loadTags()
            var tagArray = []
            for (var j = 0; j < tags.rows.length; j++) {
                tagArray.push(tags.rows.item(j))
            }
            return tagArray
        } else {
            return [{"id" : topicId, "name" : ""}]
        }
    }

    ListModel {
        id: articleListModel
    }

    /* Proxy model for RssFeedPage - to display all articles
     * instead of "shorts" (~10).
     */
    ListModel {
        id: articleListProxyModel
    }

    Component {
        id: listModeDelegate

        ListItems.Subtitled {
            text: model.title
            subText: DateUtils.formatRelativeTime(i18n, model.pubdate) + ", " + model.feed_name
            progression: false
            icon: selectIcon()
            iconFrame: true

            function selectIcon() {
                var img = model.image
                return (model.image == null || model.image == "" ) ? Qt.resolvedUrl("../rssreader64.png"): model.image
            }

            onClicked: {
                mainView.toRssPage(articleListProxyModel, model.serialNumber)
            }
        } // delegate
    } // Component

    ListView {
        id: articleList

        clip: true
        anchors.fill: parent
        visible: isShorts
        model: isShorts ? articleListModel : null

        delegate: listModeDelegate

        section {
            criteria: ViewSection.FullString
            property: "tagId"

            delegate: ListItems.Header {
                height: units.gu(5)
                text: textBySection()

                function textBySection() {
                    var s = section
                    for (var i = 0; i < articleListModel.count; i++) {
                        if (articleListModel.get(i).tagId == s) {
                            return articleListModel.get(i).tagName
                        }
                    }
                    // console.log("TEXT DON'T FOUND FOR SECTION ", s)
                    return ""
                }
            }
        } // section
    } // ListView

    ListView {
        id: articleListByTag

        clip: true
        anchors.fill: parent
        visible: !isShorts
        model: isShorts ? null : articleListProxyModel

        delegate: listModeDelegate

        section {
            criteria: ViewSection.FullString
            property: "feedId"

            delegate: ListItems.Header {
                height: units.gu(5)
                text: textBySection()

                function textBySection() {
                    var s = section
                    for (var i = 0; i < articleListProxyModel.count; i++) {
                        if (articleListProxyModel.get(i).feedId == s) {
                            return articleListProxyModel.get(i).feed_name
                        }
                    }
                    // console.log("TEXT DON'T FOUND FOR SECTION ", s)
                    return ""
                }
            }
        } // section
    } // ListView

    Label {
        id: lblEmptyBase

        text: i18n.tr("There are no articles to show")
        anchors.centerIn: parent
        visible: articleListModel.count == 0
        fontSize: "large"
    }
} // Page
