# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""RSS Reader app autopilot tests."""

from __future__ import absolute_import


import unittest

from autopilot.matchers import Eventually
from testtools.matchers import Equals, NotEquals, Is, Not

from autopilot.introspection.dbus import StateNotFoundError
from ubuntu_rssreader_app.tests import RssReaderAppTestCase

from time import sleep

class TestMainWindow(RssReaderAppTestCase):

    def setUp(self):
        super(TestMainWindow, self).setUp()
        self.assertThat(self.main_view.visible, Eventually(Equals(True)))

    def _add_feed_and_topic(self):
        """test add feed and topic"""
        #open toolbar
        toolbar = self.main_view.open_toolbar()

        #click on add reads toolbar button
        toolbar.click_button("addreadsbutton")

        #click on add feeds popover action
        addReadsPopover = self.main_view.get_action_popover()
        self.main_view.run_action(addReadsPopover, "+ Add feeds")

        #verify Append Feed  page is open
        appendpage = lambda: self.main_view.get_append_page().visible
        self.assertThat(appendpage, Eventually(Equals(True)))

        #type feed in input field
        self._input_new_feed("http://www.canonical.com/rss.xml")

        #check that feed is updated, not empty
        self.assertThat(self.main_view.get_canonical_feed_labelvisual, Eventually(Not(Is(None))))
        canonicalFeed = self.main_view.get_canonical_feed_labelvisual()
        self.assertThat(canonicalFeed.text, Eventually(Equals("Canonical")))

        #click on checkbox to confirm feed
        self.assertThat(self.main_view.get_checkbox_button, Eventually(Not(Is(None))))
        checkboxBtn = self.main_view.get_checkbox_button()
        self.pointing_device.click_object(checkboxBtn)
        self.assertThat(checkboxBtn.checked, Eventually(Equals(True)))

        #click on Next button
        self.assertThat(self.main_view.get_next_button, Eventually(Not(Is(None))))
        nextButton = self.main_view.get_next_button()
        self.pointing_device.click_object(nextButton)

        chossetopicpage = lambda: self.main_view.get_topics_page().visible
        self.assertThat(chossetopicpage, Eventually(Equals(True)))

        #add a new topic
        self._input_new_topic("CanonicalTopic")

        #select canonical topic
        self.assertThat(self.main_view.get_canonical_topic_labelvisual, Eventually(Not(Is(None))))
        canonicalTopic = self.main_view.get_canonical_topic_labelvisual()
        self.pointing_device.click_object(canonicalTopic)

        #just for timing purposes
        self.assertThat(canonicalTopic.text, Eventually(Equals("CanonicalTopic")))

    def _remove_feed(self):
        #open toolbar
        toolbar = self.main_view.open_toolbar()

        #click on back toolbar Button
        toolbar.click_button("edittopicsbutton")

        #verify we are on the Management page
        feedlistpage = lambda: self.main_view.get_feed_management_page().visible
        self.assertThat(feedlistpage, Eventually(Equals(True)))

        #select  canonical topic
        self._get_canonical_topic()

        #select  canonical feed
        self.assertThat(self.main_view.get_canonical_feed_FeedComponent, Eventually(Not(Is(None))))
        canonicalFeed = self.main_view.get_canonical_feed_FeedComponent()
        self.pointing_device.click_object(canonicalFeed)

        #verify we are on the Edit Feed page
        editFeedPage = self.main_view.get_edit_feed_page()
        self.assertThat(editFeedPage.visible, Eventually(Equals(True)))

        #open toolbar
        self.main_view.open_toolbar()

        #press delete button
        self.assertThat(self.main_view.get_delete_button, Eventually(Not(Is(None))))
        deleteButton = self.main_view.get_delete_button()
        self.pointing_device.click_object(deleteButton)

        #verify we are on the Management page
        feedlistpage = lambda: self.main_view.get_feed_management_page().visible
        self.assertThat(feedlistpage, Eventually(Equals(True)))

        #check canonical topic
        self.assertThat(self.main_view.get_canonical_topic_labelvisual, Eventually(Not(Is(None))))

        #check feed has been removed
        self.assertThat(self.main_view.get_canonical_feed_FeedComponent, Eventually(Equals(None)))

    def _remove_topic(self):
        #this been quasi converted to the sdk, but does not work as-is
        #see https://bugs.launchpad.net/ubuntu-rssreader-app/+bug/1211631

        #open toolbar
        toolbar = self.main_view.open_toolbar()

        #click on back toolbar Button
        toolbar.click_button("edittopicsbutton")

        #verify we are on the Management page
        feedlistpage = self.main_view.get_feed_management_page()
        self.assertThat(feedlistpage.visible, Eventually(Equals(True)))

        #select  canonical topic
        canonicalTopic = self._get_canonical_topic()

        topicsPage = self.main_view.get_manage_topics_page()
        self.assertThat(topicsPage, Eventually(Not(Is(None))))

        addATopicItem = lambda: self.main_view.get_add_a_topic()
        self.assertThat(addATopic, Eventually(Not(Is(None))))

        #swipe to delete canonical topic
        #we have to consider the top tab offset, then create a swipe operation across the width of the page
        startX = int(topicsPage.x + topicsPage.width * 0.20)
        stopX = int(topicsPage.x + topicsPage.width * 0.80)
        tabHeightOffset = qmlView.height - topicsPage.height + addATopicItem.height
        lineY = int(qmlView.y + canonicalTopic.y +  tabHeightOffset)

        #swipe to remove the topic
        self.pointing_device.move(startX, lineY)
        self.pointing_device.drag(startX, lineY, stopX, lineY)

        #check that it is gone
        canonicalTopic = lambda: self.main_view.get_canonical_topic()
        self.assertThat(canonicalTopic.visible, Eventually(Equals(False)))

    def _input_new_feed(self, rssFeed):
        inputField = self.main_view.get_append_feed_input_box()
        return self._safe_enter_text(inputField, rssFeed)

    def _input_new_topic(self, topic):
        inputField = self.main_view.get_new_topic_input_box()
        return self._safe_enter_text(inputField, topic)

    def _safe_enter_text(self, inputField, text, timeout = 10):
        #poll clicking for focus for lack of better method, due to screen switching animation
        #is there a property we can use instead?
        poll = 0
        while inputField.focus != True and poll < timeout:
            self.pointing_device.click_object(inputField)
            sleep(1)
            poll += 1
        self.keyboard.type(text)
        self.assertThat(inputField.text, Eventually(Equals(text)))
        self.keyboard.press_and_release("Enter")
        return inputField

    def _get_canonical_topic(self, timeout = 10):
        self.assertThat(self.main_view.get_canonical_topic_TopicComponent, Eventually(Not(Is(None))))
        canonicalTopic = self.main_view.get_canonical_topic_TopicComponent()
        self.pointing_device.click_object(canonicalTopic)

        poll = 0
        while canonicalTopic.isExpended == False and poll < timeout:
            self.pointing_device.click_object(canonicalTopic)
            sleep(1)
            poll += 1
        return canonicalTopic


    def test_add_remove_feed_and_topic(self):
        """ test add and remove feed and topic"""

        #add topic and feed
        self._add_feed_and_topic()

        #remove added Canonical topic
        #12 Aug 2013, removing topic broken for now
        #test commented until fixed, see https://bugs.launchpad.net/ubuntu-rssreader-app/+bug/1211631
        ##        self._remove_topic()

        #remove added Canonical feed
        self._remove_feed()

    @unittest.skip("Topic page broken, see https://bugs.launchpad.net/ubuntu-rssreader-app/+bug/1206248")
    def test_view_feeds(self):
        """test view feeds"""

        #-----> view all news:
        #open toolbar
        toolbar = self.main_view.open_toolbar()

        #click on add reads toolbar button
        toolbar.click_button("List view")

        #verify we are on the Feeds page
        feedlistpage = self.main_view.get_feedlist_page()
        self.assertThat(feedlistpage.visible, Eventually(Equals(True)))

        #select All articles
        allFeeds = self.main_view.get_all_feeds()
        self.assertThat(allFeeds, NotEquals("None"))
        self.pointing_device.click_object(allFeeds )

        #verify we are on the All articles page
        articlelistpage = self.main_view.get_articlelist_page()
        self.assertThat(articlelistpage.visible, Eventually(Equals(True)))

        toolbar = self.main_view.open_toolbar()

        #click on back toolbar button
        toolbar.click_button("Back")

        #verify we are on the Feeds page
        feedlistpage = lambda: self.main_view.get_feedlist_page().visible
        self.assertThat(feedlistpage, Eventually(Equals(True)))

        #-----> view individual feed:
        #add topic and feed
        self._add_feed_and_topic()

        #select Canonical feed
        canonicalFeed = lambda: self.main_view.get_canonical_feed_labelvisual()
        self.assertThat(canonicalFeed, Eventually(Not(Is(None))))
        self.pointing_device.click_object(canonicalFeed)

        #verify we are on the Canonical page
        header = lambda: self.main_view.get_header()[0].title
        self.assertThat(header, Eventually(Equals("Canonical")))

        #toggle toolbar
        toolbar = self.main_view.open_toolbar()

        #click on back toolbar button
        toolbar.click_button("Back")

        #verify we are on the Feeds page
        feedlistpage = lambda: self.main_view.get_feedlist_page().visible
        self.assertThat(feedlistpage, Eventually(Equals(True)))

        #toggle toolbar
        toolbar = self.main_view.open_toolbar()

        #click on shorts toolbar button
        toolbar.click_button("shorts")

        #verify we are on the Shorts page
        shortsPage = lambda: self.main_view.switch_to_tab_by_index(0).visible
        self.assertThat(shortsPage, Eventually(Equals(True)))

        #remove added Canonical topic
        self._remove_topic()

        #remove added Canonical feed
        self._remove_feed()
